package com.cch_lab.android.apply.sample.kitkat_transition_animation;

import com.cch_lab.android.apply.sample.kitkat_transition_animation.R;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.Color;
import android.transition.ChangeBounds;
import android.transition.Scene;
import android.transition.TransitionManager;
import android.transition.TransitionSet;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class MainActivity extends Activity implements OnClickListener {

	private LinearLayout   screenLayout;
	private LinearLayout   sceneParentViewGroup;
	private RelativeLayout sceneRootViewGroup;
	private Button         button;
	
	private Scene          scene1;
	private Scene          scene2;
	private Scene          scene3;
	private Scene          scene4;
	
	private final int SCREEN_LAYOUT_ID           = 10000;
	private final int SCENE_PARENT_VIEW_GROUP_ID = 10001;
	private final int SCENE_ROOT_VIEW_GROUP_ID   = 10002;
	private final int DROID_A_VIEW_ID            = 10003;
	private final int DROID_B_VIEW_ID            = 10004;
	private final int KITKAT_VIEW_ID             = 10005;
	private final int MESSAGE_VIEW_ID            = 10006;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		//画面のルート・レイアウト
		screenLayout = new LinearLayout(this);
		screenLayout.setId(SCREEN_LAYOUT_ID);
		screenLayout.setOrientation(LinearLayout.VERTICAL);
		screenLayout.setLayoutParams(getScreenLayoutParams());
		
		//ボタン配置
		button = new Button(this);
		button.setText("バージョンアップ");
		screenLayout.addView(button, 
						new LinearLayout.LayoutParams(
								ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT));
		button.setOnClickListener(this);
		
		//シーンルートの親ViewGroup配置
		sceneParentViewGroup =new LinearLayout(this);
		sceneParentViewGroup.setId(SCENE_PARENT_VIEW_GROUP_ID);
		sceneParentViewGroup.setOrientation(LinearLayout.VERTICAL);
		sceneParentViewGroup.setLayoutParams(getSceneParentViewGroupLayoutParams());
		screenLayout.addView(sceneParentViewGroup);
		
		//シーンのルートViewGroup配置
		//(※)初期表示内容を生成（画面下両脇にドロイド君を配置）
		sceneRootViewGroup = (RelativeLayout)createSceneRootView(1);
		sceneParentViewGroup.addView(sceneRootViewGroup);
		
		setContentView(screenLayout);
	}
	
	@Override
	public void onClick(View v) {
		//シーン表示内容１を生成（画面下両脇にドロイド君を配置）
		scene1 = crateScene(sceneParentViewGroup, createSceneRootView(1));
		
		//シーン表示内容２を生成（ドロイド君が画面中央に移動して合体）
		scene2 = crateScene(sceneParentViewGroup, createSceneRootView(2));
		
		//シーン表示内容３を生成（キットカットの巨大化）
		scene3 = crateScene(sceneParentViewGroup, createSceneRootView(3));

		//シーン表示内容４を生成 （バージョンアップ完了文字を上に向かってアニメーション）
		scene4 = crateScene(sceneParentViewGroup, createSceneRootView(4));
		
		
		//シーンとシーンの間をトランジション・アニメーション表現で表示
		doTransitionAnimation(scene1, 500, 500);
		doTransitionAnimation(scene2, 1000, 1000);
		doTransitionAnimation(scene3, 3000, 2000);
		doTransitionAnimation(scene4, 2000, 5000);
	}

	private ViewGroup createSceneRootView(int index){
		final int WRAP_CONTENT = ViewGroup.LayoutParams.WRAP_CONTENT;
		final int MATCH_PARENT = ViewGroup.LayoutParams.MATCH_PARENT;
		RelativeLayout rootViewGroup = null;
		
		//シーン表示内容１（画面下両脇にドロイド君を配置）
		if(index == 1){
			//シーンルートビューを新規生成
			rootViewGroup = new RelativeLayout(this);
			rootViewGroup.setId(SCENE_ROOT_VIEW_GROUP_ID);
			rootViewGroup.setLayoutParams(getSceneRootViewGroupLayoutParams());
			
			//シーン要素View:画面左下にドロイドＡを配置
			ImageView droidA = createImageView(DROID_A_VIEW_ID, R.drawable.droid, true);
			rootViewGroup.addView(droidA, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.ALIGN_PARENT_LEFT, RelativeLayout.ALIGN_PARENT_BOTTOM));

			//シーン要素View:画面右下にドロイドＢを配置
			ImageView droidB = createImageView(DROID_B_VIEW_ID, R.drawable.droid, true);
			rootViewGroup.addView(droidB, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.ALIGN_PARENT_RIGHT, RelativeLayout.ALIGN_PARENT_BOTTOM));
			
			//シーン要素View:画面中央にKitkatを配置（非表示）
			ImageView kitkat = createImageView(KITKAT_VIEW_ID, R.drawable.android_com_kitkat, false);
			rootViewGroup.addView(kitkat, 
					getChildViewLayoutParams(179, 253, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中下にメッセージを配置（非表示）
			TextView message = createTextView(MESSAGE_VIEW_ID, "バージョンアップ完了", false, 50, Color.RED);
			rootViewGroup.addView(message, 
					getChildViewLayoutParams(WRAP_CONTENT, WRAP_CONTENT, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.ALIGN_PARENT_BOTTOM));
		}
		
		//シーン表示内容（ドロイド君が画面中央に移動して合体）
		if(index == 2){
			//シーンルートビューを新規生成
			rootViewGroup =new RelativeLayout(this);
			rootViewGroup.setId(SCENE_ROOT_VIEW_GROUP_ID);
			rootViewGroup.setLayoutParams(getSceneRootViewGroupLayoutParams());
			
			//シーン要素View:画面中央にドロイドＡを配置
			ImageView droidA = createImageView(DROID_A_VIEW_ID, R.drawable.droid, true);
			rootViewGroup.addView(droidA, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));

			//シーン要素View:画面中央にドロイドＢを配置
			ImageView droidB = createImageView(DROID_B_VIEW_ID, R.drawable.droid, true);
			rootViewGroup.addView(droidB, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中央にKitkatを配置（非表示）（変更なし）
			ImageView kitkat = createImageView(KITKAT_VIEW_ID, R.drawable.android_com_kitkat, false);
			rootViewGroup.addView(kitkat, 
					getChildViewLayoutParams(179, 253, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中下にメッセージを配置（非表示）（変更なし）
			TextView message = createTextView(MESSAGE_VIEW_ID, "バージョンアップ完了", false, 50, Color.RED);
			rootViewGroup.addView(message, 
					getChildViewLayoutParams(WRAP_CONTENT,WRAP_CONTENT, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.ALIGN_PARENT_BOTTOM));
		}
		
		//シーン表示内容３（キットカットの巨大化）
		if(index == 3){
			//シーンルートビューを新規生成
			rootViewGroup =new RelativeLayout(this);
			rootViewGroup.setId(SCENE_ROOT_VIEW_GROUP_ID);
			rootViewGroup.setLayoutParams(getSceneRootViewGroupLayoutParams());
			
			//シーン要素View:画面中央にドロイドＡを配置（（非表示）（変更なし）
			ImageView droidA = createImageView(DROID_A_VIEW_ID, R.drawable.droid, false);
			rootViewGroup.addView(droidA, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));

			//シーン要素View:画面中央にドロイドＢを配置（非表示）（変更なし）
			ImageView droidB = createImageView(DROID_B_VIEW_ID, R.drawable.droid, false);
			rootViewGroup.addView(droidB, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中央にKitkatを配置
			ImageView kitkat = createImageView(KITKAT_VIEW_ID, R.drawable.android_com_kitkat, true);
			rootViewGroup.addView(kitkat, 
					getChildViewLayoutParams(537, 759, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中下にメッセージを配置（非表示）（変更なし）
			TextView message = createTextView(MESSAGE_VIEW_ID, "バージョンアップ完了", false, 50, Color.RED);
			rootViewGroup.addView(message, 
					getChildViewLayoutParams(WRAP_CONTENT, WRAP_CONTENT, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.ALIGN_PARENT_BOTTOM));
		}

		//シーン表示内容４ （バージョンアップ完了文字を上に向かってアニメーション）
		if(index == 4){
			//シーンルートビューを新規生成
			rootViewGroup =new RelativeLayout(this);
			rootViewGroup.setId(SCENE_ROOT_VIEW_GROUP_ID);
			rootViewGroup.setLayoutParams(getSceneRootViewGroupLayoutParams());
			
			//シーン要素View:画面中央にドロイドＡを配置（非表示）（変更なし）
			ImageView droidA = createImageView(DROID_A_VIEW_ID, R.drawable.droid, false);
			rootViewGroup.addView(droidA, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));

			//シーン要素View:画面中央にドロイドＢを配置（非表示）（変更なし）
			ImageView droidB = createImageView(DROID_B_VIEW_ID, R.drawable.droid, false);
			rootViewGroup.addView(droidB, 
					getChildViewLayoutParams(144, 144, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中央にKitkatを配置（変更なし）
			ImageView kitkat = createImageView(KITKAT_VIEW_ID, R.drawable.android_com_kitkat, true);
			rootViewGroup.addView(kitkat, 
					getChildViewLayoutParams(537, 759, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.CENTER_VERTICAL));
			
			//シーン要素View:画面中下にメッセージを配置
			TextView message = createTextView(MESSAGE_VIEW_ID, "バージョンアップ完了", true, 50, Color.RED);
			rootViewGroup.addView(message, 
					getChildViewLayoutParams(WRAP_CONTENT, WRAP_CONTENT, 
							RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.ALIGN_PARENT_TOP));
		}
		
		return (ViewGroup)rootViewGroup;
	}
	
	private Scene crateScene(ViewGroup sceneParentViewGroup, ViewGroup sceneRootViewGroup){
		return new Scene(sceneParentViewGroup, sceneRootViewGroup);
	}
	private void doTransitionAnimation(final Scene scene, long duration, long startDelay){
//		final TransitionManager transitionManager = new TransitionManager();
		final TransitionSet     transitionSet     = new TransitionSet();
		final ChangeBounds      changeBounds      = new ChangeBounds();

		//アニメーション表現設定
		transitionSet.setDuration(duration);
//		transitionSet.setStartDelay(startDelay);
		transitionSet.addTransition(changeBounds);
		
		//トランジション・アニメーション実行
		//(※)現在の表示内容から、シーン指定の表示内容に
		//　　中割りアニメーションさせながら変更します。
		sceneParentViewGroup.postDelayed(
				new Runnable() {
					@Override
					public void run() {
						TransitionManager.go(scene, transitionSet);
					}
		}, startDelay);
		
		//(※)Transition#setStartDelay(遅延ミリ秒)は、
		//　　TransitionManager.beginDelayedTransition(親ビュー)で、
		//　　シーンルートの親レイアウトの現状と遅延ミリ秒後の表示変更を
		//　　対象とするようなのでシーン一括生成の場合には、不向き？です。
	}

	private LinearLayout.LayoutParams getScreenLayoutParams(){
		LinearLayout.LayoutParams params = 
				new LinearLayout.LayoutParams(
									LinearLayout.LayoutParams.MATCH_PARENT,
									LinearLayout.LayoutParams.MATCH_PARENT);
		return params;
	}
	
	private LinearLayout.LayoutParams getSceneParentViewGroupLayoutParams(){
		LinearLayout.LayoutParams params = 
				new LinearLayout.LayoutParams(
									LinearLayout.LayoutParams.MATCH_PARENT,
									LinearLayout.LayoutParams.MATCH_PARENT);
		return params;
	}
	
	private RelativeLayout.LayoutParams getSceneRootViewGroupLayoutParams(){
		RelativeLayout.LayoutParams params = 
				new RelativeLayout.LayoutParams(
									RelativeLayout.LayoutParams.MATCH_PARENT,
									RelativeLayout.LayoutParams.MATCH_PARENT);
		return params;
	}
	
	private ImageView createImageView(int viewId, int drawableId, boolean isVisible){
		ImageView imageView = new ImageView(this);
		imageView.setId( viewId );
		imageView.setBackgroundResource( drawableId );
		imageView.setVisibility( isVisible ? View.VISIBLE : View.INVISIBLE );
		return imageView;
	}
	
	private TextView createTextView(int viewId, String message, boolean isVisible, 
										int textSize, int textColor){
		TextView textView = new TextView(this);
		textView.setId( viewId );
		textView.setText( message );
		textView.setTextSize( textSize );
		textView.setTextColor( textColor );
		textView.setVisibility( isVisible ? View.VISIBLE : View.INVISIBLE );
		return textView;
	}
	
	private RelativeLayout.LayoutParams getChildViewLayoutParams(
						int width, int height, 
						int align_horizontal, int align_vertical){
		
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(width, height);
		params.addRule(align_horizontal);
		params.addRule(align_vertical);
		return params;
	}

	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
